import Runner from './Runner';
import IRunner from './types/IRunner';
import IServiceStatus from './types/IServiceStatus';
import ISystem from './types/ISystem';
import Messenger from './types/Messenger';

const createColosseumActions = (runner: IRunner): { [Key: string]: (p: any) => Promise<any> } => ({
  add: (payload: any) => runner.add(payload.name, payload.url),
  remove: (payload: any) => runner.remove(payload.name),
  running: (payload: any) => runner.running(payload.key),
  services: async (): Promise<IServiceStatus[]> => [
    { name: 'colosseum', running: true },
    ...(await runner.getServices()),
  ],
  update: (payload: any) => runner.update(payload.name),
});

const createColosseumMessenger = (runner: IRunner) => async (messageName: string, payload: any) => {
  const actions = createColosseumActions(runner);

  try {
    const action = actions[messageName];
    if (action) {
      const response = await action(payload);
      return { success: true, response };
    }
    return { success: false, response: 'Invalid message' };
  } catch (error) {
    return { success: false, response: error.message };
  }
};

const Envoy = async (system: ISystem, installDirectory: string) => {
  await system.ensureDirectory(installDirectory);

  const message: Messenger = async (name, messageName, payload) => {
    if (name === 'colosseum') {
      return colosseumMessage(messageName, payload);
    }
    return runner.message(name, messageName, payload);
  };

  const runner = await Runner(system, message, installDirectory);
  const colosseumMessage = createColosseumMessenger(runner);

  return {
    message,
  };
};

export default Envoy;
