import { Express } from 'express';

import createClient from './helpers/createClient';
import createServer from './helpers/createServer';
import getArgs from './helpers/extractArgs';
import Poster from './types/Poster';

type ServiceFunction = (client: IClient) => Promise<void>;
type Loader = (path: string) => Promise<ServiceFunction>;
type Listener = (server: Express, port: number) => Promise<void>;

const Service = async (args: string[], poster: Poster, loader: Loader, listener: Listener) => {
  const { host, key, path, port } = getArgs(args);

  const { execute, client } = createClient(host, key, poster);
  const server = createServer(key, execute);

  const service = await loader(path);
  await service(client);

  await listener(server, Number(port));
  await client.message('colosseum', 'running', { key });
};

export default Service;
