import * as bcrypt from 'bcrypt';
import * as jwt from 'jsonwebtoken';

const hash = (value: string) =>
  new Promise<string>(resolve => {
    bcrypt.hash(value, 10, (err, result) => {
      resolve(result);
    });
  });

const compare = (adminPasswordHash: string, password: string) =>
  new Promise(resolve => {
    bcrypt.compare(password, adminPasswordHash, (err, res) => {
      resolve(res);
    });
  });

const sign = (secret: string, value: any) => jwt.sign(value, secret);

const generateToken = async (secret: string, adminPasswordHash: string, password: string) => {
  if (await compare(adminPasswordHash, password)) {
    return sign(secret, { admin: true });
  }
};

const verifyToken = (secret: string, token: string) =>
  new Promise(resolve => {
    jwt.verify(token, secret, (err, decoded) => {
      resolve(decoded);
    });
  });

export { hash, generateToken, verifyToken };
