import IExecuteResponse from '../types/IExecuteResponse';

const createHandler = () => {
  const handlers: { [Key: string]: (payload: any) => void } = {};

  const handle = (messageName: string, handler: (payload: any) => void) => {
    handlers[messageName] = handler;
  };

  const execute = async (messageName: string, payload: any): Promise<IExecuteResponse> => {
    const handler = handlers[messageName];
    if (handler) {
      try {
        const response = await Promise.resolve(handler(payload));
        return {
          error: null,
          response,
          status: 200,
        };
      } catch (ex) {
        return {
          error: ex,
          response: null,
          status: 500,
        };
      }
    } else {
      return {
        error: `No handler for message ${messageName}`,
        response: null,
        status: 400,
      };
    }
  };

  return {
    execute,
    handle,
  };
};

export default createHandler;
