import * as bodyParser from 'body-parser';
import * as express from 'express';

import IExecuteResponse from '../types/IExecuteResponse';

const createServer = (key: string, execute: (messageName: string, payload: any) => Promise<IExecuteResponse>) => {
  const server = express();

  server.use(bodyParser.json());

  server.post('/:message/:key', async (req, res) => {
    if (req.params.key === key) {
      const { error, response, status } = await execute(req.params.message, req.body);

      res.status(status);

      error ? res.send(error.stack || error) : res.json(response);
    } else {
      res.status(403).send('🖕');
    }
  });

  return server;
};

export default createServer;
