import { spawn } from 'child_process';

const start = (
  command: string,
  args: string[],
  directory: string,
  onOutput: (message: string) => void,
  onError: (message: string) => void,
  onStopped: (code: number) => void,
) => {
  let internalOnStopped: () => void;

  const process = spawn(command, args, {
    cwd: directory,
  });

  process.stdout.on('data', data => {
    if (onOutput) {
      onOutput(data.toString().trim());
    }
  });

  process.stderr.on('data', data => {
    if (onError) {
      onError(data.toString().trim());
    }
  });

  process.on('close', code => {
    if (onStopped) {
      onStopped(code);
    }
    if (internalOnStopped) {
      internalOnStopped();
    }
  });

  const stop = () =>
    new Promise<void>(resolve => {
      internalOnStopped = resolve;
      process.kill();
    });

  return {
    stop,
  };
};

const run = (
  command: string,
  args: string[],
  directory: string,
  onOutput: (message: string) => void,
  onError: (message: string) => void,
) =>
  new Promise(resolve => {
    start(command, args, directory, onOutput, onError, resolve);
  });

export { start, run };
